/***
 * Netfire deamon
 * it manages session between peer 2 peer
 *
 * Creator : wizard@KLDP.net
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <pthread.h>

#include "netfire.h"

/* define some functions for internal use */
static void *client_handler(void *arg);

void *client_handler(void *arg)
{
  int client_sockfd = ((struct ThreadArgs *)arg)->sockfd;

  struct packet_hdr_t header;
  struct login_pkt_t login;
  struct search_pkt_t search;
  int ret;

  pthread_detach(pthread_self());

  printf("Starting session...0x%x\n", client_sockfd);

  while(1)
  {
    memset(&header, 0, sizeof(struct packet_hdr_t));

    /* Waitting message from netfire client */
    ret = recv(client_sockfd, &header, sizeof(struct packet_hdr_t), 0);
    if(ret < 0)
    {
      printf("recv failed:%d\n", ret);
      close(client_sockfd);
      return NULL;
    }
    printf("-> [HEADER] %.2x %.4x %.2x\n", header.prot, header.len, header.type);

    switch(header.type)
    {
    case LOGIN_TYPE:
      ret = recv(client_sockfd, &login, sizeof(struct login_pkt_t), 0);
      if(ret < 0)
      {
	printf("recv failed:%d\n", ret);
	close(client_sockfd);
	return NULL;
      }
      printf("-> [LOGIN] %.8s %.8s\n", login.userid, login.passwd);
      /* Matching the password - FIX ME!! */
      /* find this client's IP address - FIX ME!! */
      /* Allow netfire client to join our network!!! - FIX ME!! */
      break;
    case SEARCH_TYPE:
      ret = recv(client_sockfd, &search, sizeof(struct search_pkt_t), 0);
      if(ret < 0)
      {
	printf("recv failed:%d\n", ret);
	close(client_sockfd);
	return NULL;
      }
      printf("-> [SEARCH] %.8s %.24s\n", search.userid, search.keyword);
      /* send this search request to connected netfire clients */
      break;
    default:
      ret = recv(client_sockfd, &login, sizeof(struct login_pkt_t), 0);
      if(ret < 0)
	{
	  printf("recv failed:%d\n", ret);
	  close(client_sockfd);
	  return NULL;
	}
      printf("-> [UNKNOWN] %.8s %.8s\n", login.userid, login.passwd);
      close(client_sockfd);
      return NULL;
    }
  }

  free((struct ThreadArgs *)arg);

  close(client_sockfd);

  return NULL;
}

void demonize(void)
{
  /* demonize it - FIX ME!! */
}
  
int main(int argc, char **argv)
{
  /* server and client socket descriptor */
  int serv_sockfd;
  int client_sockfd;

  /* server and client socket address */
  struct sockaddr_in server_addr;
  struct sockaddr_in client_addr;

  /* client address size */
  int client_addr_size;
  int ret;

  /* thread handle */
  struct ThreadArgs *thread_args;
  pthread_t hThread;

  /* create the server socket with TCP option */
  serv_sockfd = socket(PF_INET, SOCK_STREAM, 0);
  if(serv_sockfd == -1)
  {
    printf("socket faild:%d\n", serv_sockfd);
    exit(1);
  }

  /* clear and fill out the server address */
  memset(&server_addr, 0 , sizeof(server_addr));
  server_addr.sin_family = AF_INET;
  server_addr.sin_addr.s_addr = htonl(INADDR_ANY);
  server_addr.sin_port = htons(7880);

  /* bind the server socket to this system */
  if((ret = bind(serv_sockfd, 
	  (struct sockaddr *) &server_addr, 
	  sizeof(server_addr))) == -1)
  {
    printf("bind failed:%d\n", ret);
    exit(1);
  }

  /* listen the client until it request to serve */
  if((ret = listen(serv_sockfd, 20)) == -1)
  {
    printf("listen failed:%d\n", ret);
    exit(1);
  }

  /* accept some client */
  while(1)
  {
    printf("Waitting for client...\n");
    client_addr_size = sizeof(client_addr);
    client_sockfd = accept(serv_sockfd, 
			   (struct sockaddr *) &client_addr, 
			   &client_addr_size);
    if(client_sockfd <= 0)
    {
      printf("accept failed:%d\n", client_sockfd);
    }

    if((thread_args = (struct ThreadArgs *) malloc(sizeof(struct ThreadArgs))) == NULL)
    {
      printf("malloc failed\n");
      continue;
    }
    thread_args->sockfd = client_sockfd;

    /* create one client to process session message */
    printf("Create a thread for client...\n");
    pthread_create(&hThread, NULL, client_handler, (void *)thread_args);
  }

  return 0;
}

